package util

import groovy.transform.CompileStatic
import io.tswf.groovy.bettergroovy.scripting.v2.git.Git

@CompileStatic
class GitTags {
    public static final String DEFAULT_RELEASE_TAG_PREFIX = "release-"
    public static final String DEFAULT_DEPLOY_TAG_PREFIX = DEFAULT_RELEASE_TAG_PREFIX

    static Set<String> getDeployPrefixes() {
        return CIProperties.findListProperty("git.tag.deploy.prefixes")
            .orElse([DEFAULT_DEPLOY_TAG_PREFIX])
            .toSet()
    }

    static Set<String> getReleasePrefixes() {
        return CIProperties.findListProperty("git.tag.release.prefixes")
                .orElse([DEFAULT_RELEASE_TAG_PREFIX])
                .toSet()
    }

    static List<String> getTagsWithPrefix(Collection<String> prefixes) {
        def matchingTags = Git.tags.stream()
                .filter {tag -> prefixes.any { tag.startsWith(it) }}
                .toList()

        return matchingTags
    }

    static List<String> getReleaseTags() {
        def releasePrefixes = getReleasePrefixes()
        return getTagsWithPrefix(releasePrefixes)
    }

    static List<String> getDeployTags() {
        def deployPrefixes = getDeployPrefixes()
        return getTagsWithPrefix(deployPrefixes)
    }

    static List<String> getAllPrefixes() {
        def result = new ArrayList<String>()

        result.addAll(getReleasePrefixes())
        result.addAll(getDeployPrefixes())

        return result
    }

    static String sanitizeTagFromPrefixes(String tag, Collection<String> prefixes = getAllPrefixes()) {
        return tag.removeAll(prefixes)
    }
}
